@@ -1,14 +1,14 @@
 ##
 ## Command.tcl
 ##
-## This file contains the commands of the ined editor. Commands are
+## This file contains the commands of the Tkined editor. Commands are
 ## invoked from the pulldown menus or by the editor tools. Most
 ## commands simply call the corresponding proc for every selected
 ## object. Commands never change anything on the canvas directly.
 ## For most commands, the undo/redo buffer is modified with an
 ## inverse command.
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -30,10 +30,17 @@
 ## object just deletes the group and its members.
 ##
 
-proc Command::New { editor } {
-
-    $editor new
-    $editor filename "unknown"
+proc Command::Clear { editor } {
+    set confirm [Editor::BooleanAttribute $editor confirmClear]
+    if {$confirm} {
+	set w [$editor toplevel] 
+	set name [$editor filename]
+	set res [Dialog::confirm $w.canvas "Clear view $name?" \
+		[list clear cancel] ]
+	if {[lindex $res 0] != "clear"} return
+    }
+    $editor clear
+    $editor filename "noname.tki"
 }
 
 ##
@@ -85,7 +92,7 @@ proc Command::Open { editor {fname {}} } {
 	}
     }
 
-    $editor new
+    $editor clear
     $editor dirname  [file dirname $fname]
     $editor filename [file tail $fname]
 
@@ -156,7 +163,6 @@ proc Command::SaveAs { editor } {
 proc Command::PostScript { editor } {
 
     set w [$editor toplevel]
-
     set dir [$editor dirname]
     set fname [Dialog::fileselect $w.canvas \
 	       "Write PostScript to file:" $dir]
@@ -218,7 +224,7 @@ proc Command::Print { editor } {
 
     catch {close $f}
 
-    tkined_print $editor $w.canvas $fname
+    Editor::print $editor $w.canvas $fname
 
     catch {exec rm -f $fname}
 }
@@ -268,8 +274,16 @@ proc Command::History { editor } {
 ## to save before going down ...
 ##
 
-proc Command::Quit { editor } {
-    $editor quit
+proc Command::Close { editor } {
+    set confirm [Editor::BooleanAttribute $editor confirmExit]
+    if {$confirm} {
+	set w [$editor toplevel] 
+	set name [$editor filename]
+	set res [Dialog::confirm $w.canvas "Close view $name?" \
+		[list close cancel] ]
+	if {[lindex $res 0] != "close"} return
+    }
+    $editor delete
 }
 
 ##
@@ -350,22 +364,59 @@ proc Command::Delete { editor } {
 }
 
 ##
-## The label sub-menu contains commands to set the label contents.
+## The attribute sub-menu contains commands create, delete or 
+## edit attributes.
 ##
 
-proc Command::Label { editor what } {
-    if {$what == "attribute"} {
-	static attname
-	if {![info exists attname]} { set attname "" }
-	set w [$editor toplevel]
-	set result [Dialog::request $w.canvas "Attribute name:" \
-		[list [list Name: $attname] ] [list "set label" cancel] ]
+proc Command::Attribute { editor what } {
+    static attname attvalue
+    if {![info exists attname]}  { set attname "" }
+    if {![info exists attvalue]} { set attvalue "" }
+    set w [$editor toplevel]
+    if {$what == "delete"} {
+	set result [Dialog::request $w.canvas "Delete Attribute:" \
+		[list [list Name: $attname]] [list "delete attribute" cancel] ]
 	if {[lindex $result 0] == "cancel"} return
-	set attname [lindex $result 1]
-	set what $attname
+        set attname [lindex $result 1]
+	set attvalue ""
+    } else {
+	set result [Dialog::request $w.canvas "Edit Attribute:" \
+		[list [list Name: $attname] [list Value: $attvalue]] \
+		[list "set value" cancel] ]
+	if {[lindex $result 0] == "cancel"} return
+        set attname [lindex $result 1]
+	set attvalue [lindex $result 2]
+    }
+
+    foreach id [$editor selection] {
+	catch {$id attribute $attname $attvalue}
+    }
+}
+
+##
+## The label command selects an attribute for the label.
+##
+
+proc Command::Label { editor } {
+    static attname
+    if {![info exists attname]} { set attname "address" }
+    set w [$editor toplevel]
+    set result [Dialog::request $w.canvas "Select an Attribute as a Label:" \
+	    [list [list Name: $attname entry 30] ] \
+	    [list accept "no label" cancel] ]
+    switch [lindex $result 0] {
+	cancel {
+	    return
+	}
+	"no label" {
+	    set attname clear
+	}
+	default {
+	    set attname [lindex $result 1]
+	}	
     }
     foreach id [$editor selection] {
-	catch {$id label $what}
+	catch {$id label $attname}
     }
 }
 
@@ -402,11 +453,21 @@ proc Command::SelectMember { editor } {
     }
 }
 
-proc Command::SelectType { editor type } {
-    foreach id [$editor retrieve] {
-        if {[$id type] == $type} {
-	    $id select
-        }
+proc Command::SelectType { editor } {
+
+    static type
+    if {![info exists type]}  { set type  "NODE" }
+
+    set w [$editor toplevel]
+    set result [Dialog::request $w.canvas "Select objects by type." \
+	         [list [list Type: dummy option NODE NETWORK LINK GROUP \
+		       TEXT IMAGE REFERENCE STRIPCHART BARCHART GRAPH] ] \
+		 [list select cancel] ]
+    if {[lindex $result 0] == "cancel"} return
+
+    set type  [lindex $result 1]
+    foreach id [$editor retrieve $type] {
+	$id select
     }
 }
 
@@ -417,14 +478,15 @@ proc Command::SelectName { editor } {
     if {![info exists type]}  { set type  "glob" }
 
     set w [$editor toplevel]
-    set regex [Dialog::request $w.canvas "Select objects by name." {} \
-	        [list [list Expression: $regex] \
+    set result [Dialog::request $w.canvas "Select objects by name." \
+	        [list [list Expression: $regex entry 20] \
 		      [list Type: $type radio regexp glob] ] \
 		[list select cancel] ]
-    if {[lindex $regex 0] == "cancel"} return
 
-    set type  [lindex $regex 2]
-    set regex [lindex $regex 1]
+    set regex [lindex $result 1]
+    set type  [lindex $result 2]
+
+    if {[lindex $result 0] == "cancel"} return
 
     # test if the regex is valid
     if {[catch {lsearch -$type foobar $regex} res]} {
@@ -450,14 +512,15 @@ proc Command::SelectAddress { editor } {
     if {![info exists type]}  { set type  "glob" }
 
     set w [$editor toplevel]
-    set regex [Dialog::request $w.canvas "Select objects by address." {} \
-	        [list [list Expression: $regex] \
+    set result [Dialog::request $w.canvas "Select objects by address." \
+	        [list [list Expression: $regex entry 20] \
 		      [list Type: $type radio regexp glob] ] \
 		[list select cancel] ]
-    if {[lindex $regex 0] == "cancel"} return
 
-    set type  [lindex $regex 2]
-    set regex [lindex $regex 1]
+    set regex [lindex $result 1]
+    set type  [lindex $result 2]
+
+    if {[lindex $result 0] == "cancel"} return
 
     # test if the regex is valid
     if {[catch {lsearch -$type foobar $regex} res]} {
@@ -483,14 +546,15 @@ proc Command::SelectLabel { editor } {
     if {![info exists type]}  { set type  "glob" }
 
     set w [$editor toplevel]
-    set regex [Dialog::request $w.canvas "Select objects by current label." \
-	        [list [list Expression: $regex] \
+    set result [Dialog::request $w.canvas "Select objects by current label." \
+	        [list [list Expression: $regex entry 20] \
 		      [list Type: $type radio regexp glob] ] \
 		[list select cancel] ]
-    if {[lindex $regex 0] == "cancel"} return
 
-    set type  [lindex $regex 2]
-    set regex [lindex $regex 1]
+    set regex [lindex $result 1]
+    set type  [lindex $result 2]
+
+    if {[lindex $result 0] == "cancel"} return
 
     # test if the regex is valid
     if {[catch {lsearch -$type foobar $regex} res]} {
@@ -501,12 +565,9 @@ proc Command::SelectLabel { editor } {
 
     foreach id [$editor retrieve] {
 	foreach item [$id items] {
-	    set tags [$w.canvas gettags $item]
-	    if {[lsearch $tags LABEL] >= 0} {
-		set label [lindex [$w.canvas itemconfigure $item -text] 4]
-		if {[lsearch -$type $label $regex] >= 0} {
-		    $id select
-		}
+	    set label [$w.canvas itemcget label$item -text]
+             if {[lsearch -$type $label $regex] >= 0} {
+		$id select
 	    }
 	}
     }
@@ -684,13 +745,18 @@ proc Command::RemoveGroup { editor } {
     foreach id [$editor selection] {
 	if {[$id parent] != ""} {
 	    lappend group([$id parent]) $id
+	    catch {
+		foreach l [$id links] {
+		    if {[$l parent] != ""} {
+			lappend group([$l parent]) $l
+		    }
+		}
+	    }
 	}
     }
-    if {![info exists group]} return
     foreach g [array names group] {
-	set mem [$g member]
 	set new ""
-	foreach id $mem {
+	foreach id [$g member] {
 	    if {[lsearch -exact $group($g) $id] < 0} {
 		lappend new $id
 	    }
@@ -783,19 +849,51 @@ proc Command::Color { editor colorname } {
 }
 
 ##
-## Here are all commands used in the view menu.
+## Here are all commands used in the options menu.
 ##
 
-proc Command::HideShow { editor } {
+proc Command::ToggleToolBox { editor } {
     set w [$editor toplevel]
-    if {[lsearch [pack slaves $w] $w.tools] < 0} {
-	$w.menu.view.m entryconfigure last -label "Hide Toolbox" \
-	    -accelerator "  H"
-	pack $w.tools -side left -fill y -before $w.menu
+    global showToolbox$w newToolbox$w
+    if {[set showToolbox$w]} {
+	if {[set newToolbox$w]} {
+	    foreach child [winfo children $w.tools] {
+		pack configure $child -side left -expand true
+		if {[winfo class $child] == "Label"} {pack forget $child}
+	    }
+	    pack $w.tools -side top -fill x -after $w.menu -padx 5 -pady 5
+	} else {
+	    foreach child [winfo children $w.tools] {
+		pack configure $child -side top -expand false
+	    }
+	    pack $w.tools.dummy -side top -pady 1 -ipadx 5 \
+		-before $w.tools.select 
+	    pack $w.tools -side left -fill y -before $w.menu -padx 0 -pady 0
+	}
     } else {
-	$w.menu.view.m entryconfigure last -label "Show Toolbox" \
-	    -accelerator "  S"
-	Tool::Alter $editor
 	pack forget $w.tools
+        Tool::Select $editor
+    }
+}
+
+proc Command::LockEditor { editor } {
+    set w [$editor toplevel]
+    global lockEditor$w showToolbox$w
+    if {[set showToolbox$w]} {
+	set showToolbox$w 0
+	Command::ToggleToolBox $editor
+    }
+    if {[set lockEditor$w]} {
+	if {[set showToolbox$w]} {
+	    set showToolbox$w 0
+	    Command::ToggleToolBox $editor
+	}
+	Editor::lock $editor
+    } else {
+	if {! [set showToolbox$w]} {
+	    set showToolbox$w 1
+	    Command::ToggleToolBox $editor
+	}
+	Editor::unlock $editor
     }
 }
